# -*- coding=utf-8 -*-
from aptdaemon import client
from aptdaemon.errors import NotAuthorizedError, TransactionFailed, AptDaemonError
from gi.repository import GObject
import socket
import sys
import aptdaemon.loop
import apt_pkg
import string
import threading
from collections import namedtuple
import unittest
import apt
import aptdaemon.core
import os
port=8688

if len(sys.argv) > 1:
        port = int(sys.argv[1])


address = ('127.0.0.1', port)
s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)

def sendMsg(o, msg):
        s.sendto(msg.encode(), address)

class AptAction(GObject.GObject):
        __gsignals__ = {
                'sendMsg': (GObject.SIGNAL_RUN_FIRST, None,
                              (GObject.TYPE_STRING,))
                }
        def __init__(self):
                GObject.GObject.__init__(self)
                self.apt_client = client.AptClient()
                self.connect('sendMsg', sendMsg)
                self.cache = apt.Cache()
                Options = namedtuple("Options", "dummy")
                opt = opt = Options(True)
                self.daemon = aptdaemon.core.AptDaemon(opt, connect=False)
        def init_data(self):
                self.transaction = None
                self.action = ""
                self.packageName = ""
                self.status = ""
                self.bytes_done = ""
                self.bytes_total = ""
                self.speed = ""
                self.progress = ""
                self.pause = ""
                self.cancel = ""
                self.error_details = ""

        def install(self, pkgs):
                try:
                        print("install Pkgs")
                        self.init_data()
                        self.packageName = pkgs[0]
                        self.action = "install"
                        self.daemon._check_package_names(pkgs)
                        self.transaction = self.apt_client.install_packages(pkgs)
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        self.transaction.connect("progress-changed", self.change_progress)
                        self.transaction.connect("progress-details-changed", self.progress_details)
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, AptDaemonError, Exception) as e:
                        #print(e)
                        if e._dbus_error_name == "org.debian.apt":
                                self.error_details = "failed"
                                msg = self.action + ";"+self.packageName +";" +self.error_details + ";" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                                self.emit('sendMsg', msg)
                                print(msg)
                        pass
                if self.error_details == "":
                        print(self.error_details)
                        aptdaemon.loop.mainloop.run()
                        
        def remove(self, pkgs):
                try:
                        self.init_data()
                        self.action = "remove"
                        self.packageName = pkgs[0]
                        self.daemon._check_package_names(pkgs)
                        self.transaction = self.apt_client.remove_packages(pkgs)
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        self.transaction.connect("progress-changed", self.change_progress)
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, AptDaemonError, Exception) as e:
                        #print(e)
                        if e._dbus_error_name == "org.debian.apt":
                                self.error_details = "failed"
                                msg = self.action + ";"+self.packageName +";" +self.error_details + ";" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                                self.emit('sendMsg', msg)
                                print(msg)
                        pass
                if self.error_details == "":
                        print(self.error_details)
                        aptdaemon.loop.mainloop.run()
                        
        def upgrade_pkg(self, pkgs):
                try:
                        print("upgrade_pkg Pkgs")
                        self.init_data()
                        self.action = "upgrade"
                        self.packageName = pkgs[0]
                        self.daemon._check_package_names(pkgs)
                        self.transaction = self.apt_client.upgrade_packages(pkgs)
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        self.transaction.connect("progress-changed", self.change_progress)
                        self.transaction.connect("progress-details-changed", self.progress_details)
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, AptDaemonError, Exception) as e:
                        if e._dbus_error_name == "org.debian.apt":
                                self.error_details = "failed"
                                msg = self.action + ";"+self.packageName +";" +self.error_details + ";" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                                self.emit('sendMsg', msg)
                                print(msg)
                        pass
                
                if self.error_details == "":
                        aptdaemon.loop.mainloop.run()

        def cancell(self):
                self.action = "cancel"
                if self.transaction:
                        self.transaction.cancel(self.reply_handler, self.error_handler)
                        self.transaction = None


        def  reply_handler(self):
                print("reply_handler")
        def error_handler(self, error):
                print("error_handler")
                print(error)

        def pausee(self):
                self.action = "pause"
                if self.transaction:
                        self.transaction.cancel(self.reply_handler, self.error_handler)
                        self.transaction = None

        def add_repository(self, args):
                print("add_repository")
                try:
                        print("add_repository")
                        self.init_data()
                        self.action = "add_repository"
                        argLen = len(args)
                        src_type = ""
                        uri = ""
                        dist = ""
                        comps = []
                        count = 0
                        if argLen >= 3:
                                src_type = args[0]
                                uri = args[1]
                                dist = args[2]
                                for item in args[3:]:
                                        comps.append(item)
                        else:
                                return

                        #print(src_type)
                        #print(uri)
                        #print(dist)
                        #print(comps)
                        self.transaction = self.apt_client.add_repository(src_type, uri, dist, comps)
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, Exception) as e:
                        print(e)
                        pass
                aptdaemon.loop.mainloop.run()
                
        def update_cache(self):
                print("update_cache")
                try:
                        self.init_data()
                        self.action = "update"
                        repo_type = ""
                        
                        repo_local_path = '/etc/apt/sources.list.d/local-cossoftwarecenter.list'
                        repo_network_path = '/etc/apt/sources.list.d/network-cossoftwarecenter.list'
                        
                        result = os.path.isfile(repo_local_path)
                        if result == True:
                                data = ""
                                file_obj = open(repo_local_path)
                                try:
                                        data = file_obj.read()
                                finally:
                                        file_obj.close()
                                if len(data) == 0:
                                       repo_type = "network"
                                else:
                                        repo_type = "local"
                        else:
                                repo_type = "network"
                        
                        if repo_type == "local":
                                self.transaction = self.apt_client.update_cache(repo_local_path)
                                
                        if repo_type == "network":
                                #self.transaction = self.apt_client.update_cache(repo_network_path)
                                self.transaction = self.apt_client.update_cache()
                        
                        
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        #self.transaction.connect("progress-changed", self.change_progress)
                        #self.transaction.connect("progress-details-changed", self.progress_details)
                        self.transaction.connect("progress-download-changed", self.progress_download)                         
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, Exception) as e:
                        print(e)
                        pass
                aptdaemon.loop.mainloop.run()

        def clean(self):
                print("clean_cache")
                try:
                        self.init_data()
                        self.action = "clean"
                        self.transaction = self.apt_client.clean()
                        self.transaction.set_allow_unauthenticated(allow_unauthenticated=2)
                        self.transaction.connect("progress-changed", self.change_progress)
                        self.transaction.connect("finished", self.finished)
                        self.transaction.connect("error", self.on_error)
                        self.transaction.run()
                except(NotAuthorizedError, TransactionFailed, Exception) as e:
                        print(e)
                        pass
                aptdaemon.loop.mainloop.run()  
                
        def progress_download(self, transaction, uri, status, desc, full_size,
                             downloaded, message):
                try:
                        print('########')
                        print(downloaded)
                        print(full_size)
                        msg = self.action + ";"+self.packageName + ";" + "inprogress;" + str(downloaded) + ";"+ str(full_size)+";"+";"+";"+ "yes;" +"yes;" + uri;
                        self.emit('sendMsg', msg)
                        print(msg)
                        #print(status)
                        #print(message) 
                except(Exception) as e:
                        print("##############")
               
                
        def progress_details(self, transaction, items_done, items_total,
                             bytes_done, bytes_total, speed, eta):
                if bytes_total and speed:
                        self.bytes_done = (("%sB") %apt_pkg.size_to_str(bytes_done))
                        self.bytes_total = (("%sB") %apt_pkg.size_to_str(bytes_total))
                        self.speed = (("%sB/s") %apt_pkg.size_to_str(speed))

        def change_progress(self, tr, pr):
                #pkgs = [];
                #pkgs.extend(tr.packages[0])
                #pkgs.extend(tr.packages[1])
                #pkgs.extend(tr.packages[2])
                #pkgs.extend(tr.packages[3])
                #pkgs.extend(tr.packages[4])
                if pr < 45 and pr > 0:
                        msg = self.action + ";"+self.packageName + ";" + "inprogress;" + self.bytes_done + ";"+self.bytes_total+";"+self.speed + ";"+ format(pr, '03d') + ";"+ "yes;" +"yes;";
                        self.emit('sendMsg', msg)
                        print(msg) 
                elif pr >= 45 and pr <= 100:
                        msg = self.action + ";"+self.packageName + ";" + "inprogress;" + self.bytes_done + ";"+self.bytes_total+";"+self.speed + ";"+ format(pr, '03d') + ";"+ "no;" +"no;";
                        self.emit('sendMsg', msg)
                        print(msg) 
                       

        def finished(self, tr, exit_status):
                print("finished")
                print(exit_status)
                #pkgs = [];
                #pkgs.extend(tr.packages[0])
                #pkgs.extend(tr.packages[1])
                #pkgs.extend(tr.packages[2])
                #pkgs.extend(tr.packages[3])
                #pkgs.extend(tr.packages[4])
                if exit_status == 'exit-cancelled':
                        if self.action == 'cancel':
                                msg = self.action + ";"+self.packageName +";" +"success;" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                        if self.action == 'pause':
                                msg = self.action + ";"+self.packageName +";" +"success;" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                        self.emit('sendMsg', msg)
                        print(msg)
                if exit_status == 'exit-success':
                        msg = self.action + ";"+self.packageName +";" +"success;" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                        print(msg)
                        self.emit('sendMsg', msg)
                if exit_status == 'exit-failed':
                        if self.error_details == "downloadFailed":
                                msg = self.action + ";"+self.packageName +";" +self.error_details + ";" + ";"+";"+ ";"+ ";"+ "no;" +"yes;"
                        else:
                                msg = self.action + ";"+self.packageName +";" +self.error_details + ";" + ";"+";"+ ";"+ ";"+ "no;" +"no;"
                        self.emit('sendMsg', msg)
                        print(msg)
                aptdaemon.loop.mainloop.quit()

        def on_error(self, tr, error_code, error_details):
                print(error_code)
                if error_code == 'error-not-authorized':
                        self.error_details = "notAuthorized"
                elif error_code == 'error-package-not-installed':
                        self.error_details = "notInstalled"
                elif error_code == 'error-package-already-installed':
                        self.error_details = "alreadyInstalled"
                elif error_code == 'error-package-download-failed':
                        self.error_details = "downloadFailed"
                elif error_code == 'error-package-uptodate':
                        self.error_details = "uptodate"
                elif error_code == 'error-repo-download-failed':
                        self.error_details = "repoDownloadFailed"
                else:
                        self.error_details = "failed"
                print(self.error_details)

aptAction = AptAction()

def analyzeCmd(cmd):
        cmdArr = cmd.split()
        if len(cmdArr) >= 1:
                c = cmdArr[0]
                args = cmdArr[1:]
                if 'install' == c:
                        aptAction.install(args)
                elif 'remove' == c:
                        aptAction.remove(args)
                elif 'upgrade' == c:
                        aptAction.upgrade_pkg(args)
                elif 'add_repository' == c:
                    aptAction.add_repository(args)
                elif 'update' == cmd.split()[0]:
                        aptAction.update_cache()
                elif 'clean' == cmd.split()[0]:
                        aptAction.clean()                    

                    


while(True):
        cmd = input()
        if len(cmd) > 0:
                if 'exit' == cmd.split()[0]:
                        break;
                if 'cancel' == cmd.split()[0]:
                        aptAction.cancell()
                if 'pause' == cmd.split()[0]:
                        aptAction.pausee()                     
                else :
                        myThread = threading.Thread(target = analyzeCmd, args=(cmd, ))
                        myThread.start()






